@@ -13,7 +13,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import json
+from oslo_serialization import jsonutils
+import six
 
 from blazar import context
 from blazar import exceptions
@@ -21,7 +22,7 @@ from blazar import exceptions
 
 def ctx_from_headers(headers):
     try:
-        service_catalog = json.loads(headers['X-Service-Catalog'])
+        service_catalog = jsonutils.loads(headers['X-Service-Catalog'])
     except KeyError:
         raise exceptions.ServiceCatalogNotFound()
     except TypeError:
@@ -34,5 +35,5 @@ def ctx_from_headers(headers):
         service_catalog=service_catalog,
         user_name=headers['X-User-Name'],
         project_name=headers['X-Project-Name'],
-        roles=map(unicode.strip, headers['X-Roles'].split(',')),
+        roles=list(map(six.text_type.strip, headers['X-Roles'].split(','))),
     )
