@@ -14,36 +14,27 @@
 # limitations under the License.
 
 import eventlet
+eventlet.monkey_patch(
+    os=True, select=True, socket=True, thread=True, time=True)
+
 import flask
 from keystonemiddleware import auth_token
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_middleware import debug
+from stevedore import enabled
 from werkzeug import exceptions as werkzeug_exceptions
 
-from blazar.api.v1.oshosts import v1_0 as host_api_v1_0
 from blazar.api.v1 import utils as api_utils
-from blazar.api.v1 import v1_0 as api_v1_0
 
 
 LOG = logging.getLogger(__name__)
 
 CONF = cfg.CONF
 
-CONF.import_opt('os_auth_host', 'blazar.config')
-CONF.import_opt('os_auth_port', 'blazar.config')
-CONF.import_opt('os_auth_protocol', 'blazar.config')
-CONF.import_opt('os_admin_username', 'blazar.config')
-CONF.import_opt('os_admin_password', 'blazar.config')
-CONF.import_opt('os_admin_project_name', 'blazar.config')
-CONF.import_opt('os_auth_version', 'blazar.config')
 CONF.import_opt('log_exchange', 'blazar.config')
 
 
-eventlet.monkey_patch(
-    os=True, select=True, socket=True, thread=True, time=True)
-
-
 def make_json_error(ex):
     if isinstance(ex, werkzeug_exceptions.HTTPException):
         status_code = ex.code
@@ -77,17 +68,22 @@ def make_app():
 
     app.route('/', methods=['GET'])(version_list)
     app.route('/versions', methods=['GET'])(version_list)
-    app.register_blueprint(api_v1_0.rest, url_prefix='/v1')
 
     LOG.debug("List of plugins: %s", cfg.CONF.manager.plugins)
-    # TODO(sbauza) : Change this whole crap by removing hardcoded values and
-    #   maybe using stevedore for achieving this
-    if (cfg.CONF.manager.plugins
-            and 'physical.host.plugin' in cfg.CONF.manager.plugins):
-        app.register_blueprint(host_api_v1_0.rest, url_prefix='/v1/os-hosts')
 
-    for code in werkzeug_exceptions.default_exceptions.iterkeys():
-        app.error_handler_spec[None][code] = make_json_error
+    plugins = cfg.CONF.manager.plugins + ['leases']
+    extension_manager = enabled.EnabledExtensionManager(
+        check_func=lambda ext: ext.name in plugins,
+        namespace='blazar.api.v1.extensions',
+        invoke_on_load=False
+        )
+
+    for ext in extension_manager.extensions:
+        bp = ext.plugin()
+        app.register_blueprint(bp, url_prefix=bp.url_prefix)
+
+    for code in werkzeug_exceptions.default_exceptions:
+        app.register_error_handler(code, make_json_error)
 
     if cfg.CONF.debug and not cfg.CONF.log_exchange:
         LOG.debug('Logging of request/response exchange could be enabled '
@@ -96,15 +92,6 @@ def make_app():
     if cfg.CONF.log_exchange:
         app.wsgi_app = debug.Debug.factory(app.config)(app.wsgi_app)
 
-    app.wsgi_app = auth_token.filter_factory(
-        app.config,
-        auth_host=cfg.CONF.os_auth_host,
-        auth_port=cfg.CONF.os_auth_port,
-        auth_protocol=cfg.CONF.os_auth_protocol,
-        admin_user=cfg.CONF.os_admin_username,
-        admin_password=cfg.CONF.os_admin_password,
-        admin_tenant_name=cfg.CONF.os_admin_project_name,
-        auth_version=cfg.CONF.os_auth_version,
-    )(app.wsgi_app)
+    app.wsgi_app = auth_token.filter_factory(app.config)(app.wsgi_app)
 
     return app
