@@ -26,6 +26,7 @@ from blazar.db import exceptions as db_exceptions
 from blazar import exceptions as ex
 from blazar.i18n import _
 from blazar.manager import exceptions as manager_exceptions
+from blazar.utils.openstack import exceptions as opst_exceptions
 
 LOG = logging.getLogger(__name__)
 
@@ -33,13 +34,18 @@ LOG = logging.getLogger(__name__)
 class Rest(flask.Blueprint):
     """REST helper class."""
 
-    def get(self, rule, status_code=200):
-        return self._mroute('GET', rule, status_code)
+    def __init__(self, *args, **kwargs):
+        super(Rest, self).__init__(*args, **kwargs)
+        self.url_prefix = kwargs.get('url_prefix', None)
+        self.routes_with_query_support = []
 
-    def post(self, rule, status_code=202):
+    def get(self, rule, status_code=200, query=False):
+        return self._mroute('GET', rule, status_code, query=query)
+
+    def post(self, rule, status_code=201):
         return self._mroute('POST', rule, status_code)
 
-    def put(self, rule, status_code=202):
+    def put(self, rule, status_code=200):
         return self._mroute('PUT', rule, status_code)
 
     def delete(self, rule, status_code=204):
@@ -55,13 +61,13 @@ class Rest(flask.Blueprint):
         """Routes REST method and its params to the actual request."""
         status = options.pop('status_code', None)
         file_upload = options.pop('file_upload', False)
+        query = options.pop('query', False)
 
         def decorator(func):
             endpoint = options.pop('endpoint', func.__name__)
 
             def handler(**kwargs):
                 LOG.debug("Rest.route.decorator.handler, kwargs=%s", kwargs)
-
                 _init_resp_type(file_upload)
 
                 # update status code
@@ -71,6 +77,10 @@ class Rest(flask.Blueprint):
                 if flask.request.method in ['POST', 'PUT']:
                     kwargs['data'] = request_data()
 
+                if flask.request.endpoint in self.routes_with_query_support:
+                    params = {k: v for k, v in get_request_args().items()}
+                    kwargs['query'] = params
+
                 with context.ctx_from_headers(flask.request.headers):
                     try:
                         return func(**kwargs)
@@ -80,6 +90,8 @@ class Rest(flask.Blueprint):
                         # Get the exception from manager and common exceptions
                         cls = getattr(manager_exceptions, e.exc_type,
                                       getattr(ex, e.exc_type, None))
+                        cls = cls or getattr(opst_exceptions, e.exc_type,
+                                             getattr(ex, e.exc_type, None))
                         if cls is not None:
                             return render_error_message(cls.code, e.value,
                                                         cls.code)
@@ -103,6 +115,9 @@ class Rest(flask.Blueprint):
                     except Exception as e:
                         return internal_error(500, 'Internal Server Error', e)
 
+            if query:
+                self.routes_with_query_support.append(
+                    '.'.join([self.name, endpoint]))
             self.add_url_rule(rule, endpoint, handler, **options)
             self.add_url_rule(rule + '.json', endpoint, handler, **options)
 
@@ -160,7 +175,7 @@ def render(result=None, response_type=None, status=None, **kwargs):
                       _("Content type '%s' isn't supported") % response_type)
         return
 
-    body = serializer.dumps(result)
+    body = serializer.dump_as_bytes(result)
 
     response_type = str(response_type)
 
@@ -202,8 +217,8 @@ def get_request_args():
 
 def abort_and_log(status_code, descr, exc=None):
     """Process occurred errors."""
-    LOG.error(_("Request aborted with status code %(code)s and "
-                "message '%(msg)s'"), {'code': status_code, 'msg': descr})
+    LOG.error("Request aborted with status code %(code)s and "
+              "message '%(msg)s'", {'code': status_code, 'msg': descr})
 
     if exc is not None:
         LOG.error(traceback.format_exc())
@@ -227,8 +242,8 @@ def render_error_message(error_code, error_message, error_name):
 
 def internal_error(status_code, descr, exc=None):
     """Called if internal error occurred."""
-    LOG.error(_("Request aborted with status code %(code)s "
-                "and message '%(msg)s'"), {'code': status_code, 'msg': descr})
+    LOG.error("Request aborted with status code %(code)s "
+              "and message '%(msg)s'", {'code': status_code, 'msg': descr})
 
     if exc is not None:
         LOG.error(traceback.format_exc())
@@ -247,9 +262,9 @@ def bad_request(error):
 
     LOG.debug("Validation Error occurred: error_code=%(code)s, "
               "error_message=%(msg)s, error_name=%(name)s",
-              {'code': error.code, 'msg': error.message, 'name': error.code})
+              {'code': error.code, 'msg': str(error), 'name': error.code})
 
-    return render_error_message(error.code, error.message, error.code)
+    return render_error_message(error.code, str(error), error.code)
 
 
 def not_found(error):
@@ -259,6 +274,6 @@ def not_found(error):
 
     LOG.debug("Not Found exception occurred: error_code=%(code)s, "
               "error_message=%(msg)s, error_name=%(name)s",
-              {'code': error.code, 'msg': error.message, 'name': error.code})
+              {'code': error.code, 'msg': str(error), 'name': error.code})
 
-    return render_error_message(error.code, error.message, error.code)
+    return render_error_message(error.code, str(error), error.code)
