@@ -28,7 +28,7 @@ interface.
               `sqlalchemy` is the only supported backend right now.
 
 :sql_connection:  string specifying the sqlalchemy connection to use, like:
-                  `sqlite:///var/lib/climate/climate.sqlite`.
+                  `sqlite:///var/lib/blazar/blazar.sqlite`.
 
 """
 
@@ -107,14 +107,35 @@ def get_reservations_by_host_id(host_id, start_date, end_date):
     return IMPL.get_reservations_by_host_id(host_id, start_date, end_date)
 
 
-def get_free_periods(resource_id, start_date, end_date, duration):
+def get_reservations_by_host_ids(host_ids, start_date, end_date):
+    return IMPL.get_reservations_by_host_ids(host_ids, start_date, end_date)
+
+
+def get_reservation_allocations_by_host_ids(host_ids, start_date, end_date,
+                                            lease_id=None,
+                                            reservation_id=None):
+    return IMPL.get_reservation_allocations_by_host_ids(host_ids,
+                                                        start_date, end_date,
+                                                        lease_id,
+                                                        reservation_id)
+
+
+def get_plugin_reservation(resource_type, resource_id):
+    return IMPL.get_plugin_reservation(resource_type, resource_id)
+
+
+def get_free_periods(resource_id, start_date, end_date, duration,
+                     resource_type='host'):
     """Returns a list of free periods."""
-    return IMPL.get_free_periods(resource_id, start_date, end_date, duration)
+    return IMPL.get_free_periods(resource_id, start_date, end_date, duration,
+                                 resource_type=resource_type)
 
 
-def get_full_periods(resource_id, start_date, end_date, duration):
-    """Returns a list of full periods."""
-    return IMPL.get_full_periods(resource_id, start_date, end_date, duration)
+def get_reserved_periods(resource_id, start_date, end_date, duration,
+                         resource_type='host'):
+    """Returns a list of reserved periods."""
+    return IMPL.get_reserved_periods(resource_id, start_date, end_date,
+                                     duration, resource_type=resource_type)
 
 
 def reservation_ratio(resource_id, start_date, end_date):
