@@ -15,6 +15,7 @@
 
 import datetime
 
+import ddt
 import eventlet
 import mock
 from oslo_config import cfg
@@ -84,6 +85,7 @@ class FakeLeaseStatus(object):
         return decorator
 
 
+@ddt.ddt
 class ServiceTestCase(tests.TestCase):
     def setUp(self):
         super(ServiceTestCase, self).setUp()
@@ -143,6 +145,7 @@ class ServiceTestCase(tests.TestCase):
                                         'resource_id': '111',
                                         'resource_type': 'virtual:instance',
                                         'status': 'FAKE PROGRESS'}],
+                      'status': status.LeaseStatus.PENDING,
                       'start_date': datetime.datetime(2013, 12, 20, 13, 00),
                       'end_date': datetime.datetime(2013, 12, 20, 15, 00),
                       'trust_id': 'exxee111qwwwwe'}
@@ -218,7 +221,7 @@ class ServiceTestCase(tests.TestCase):
         self.assertEqual(actions, self.manager._setup_actions())
 
     def test_no_events(self):
-        events = self.patch(self.db_api, 'event_get_first_sorted_by_filters')
+        events = self.patch(self.db_api, 'event_get_all_sorted_by_filters')
         event_update = self.patch(self.db_api, 'event_update')
         events.return_value = None
 
@@ -227,23 +230,29 @@ class ServiceTestCase(tests.TestCase):
         self.assertFalse(event_update.called)
 
     def test_event_success(self):
-        events = self.patch(self.db_api, 'event_get_first_sorted_by_filters')
+        events = self.patch(self.db_api, 'event_get_all_sorted_by_filters')
         event_update = self.patch(self.db_api, 'event_update')
-        events.return_value = {'id': '111-222-333',
-                               'time': self.good_date}
+        events.return_value = [{'id': '111-222-333',
+                                'lease_id': 'lease_id1',
+                                'time': self.good_date},
+                               {'id': '444-555-666',
+                                'lease_id': 'lease_id2',
+                                'time': self.good_date}]
         self.patch(eventlet, 'spawn_n')
 
         self.manager._event()
 
-        event_update.assert_called_once_with(
-            '111-222-333', {'status': status.event.IN_PROGRESS})
+        event_update.assert_has_calls([
+            mock.call('111-222-333', {'status': status.event.IN_PROGRESS}),
+            mock.call('444-555-666', {'status': status.event.IN_PROGRESS})])
 
     def test_event_spawn_fail(self):
-        events = self.patch(self.db_api, 'event_get_first_sorted_by_filters')
+        events = self.patch(self.db_api, 'event_get_all_sorted_by_filters')
         event_update = self.patch(self.db_api, 'event_update')
         self.patch(eventlet, 'spawn_n').side_effect = Exception
-        events.return_value = {'id': '111-222-333',
-                               'time': self.good_date}
+        events.return_value = [{'id': '111-222-333',
+                                'lease_id': self.lease_id,
+                                'time': self.good_date}]
 
         self.manager._event()
 
@@ -251,6 +260,23 @@ class ServiceTestCase(tests.TestCase):
             mock.call('111-222-333', {'status': status.event.IN_PROGRESS}),
             mock.call('111-222-333', {'status': status.event.ERROR})])
 
+    def test_event_pass(self):
+        events = self.patch(self.db_api, 'event_get_all_sorted_by_filters')
+        events.return_value = [{'id': '111-222-333',
+                                'lease_id': self.lease_id,
+                                'time': self.good_date}]
+
+        self.lease_get = self.patch(self.db_api, 'lease_get')
+        lease = self.lease.copy()
+        lease.update({'status': status.LeaseStatus.CREATING})
+        self.lease_get.return_value = lease
+
+        event_update = self.patch(self.db_api, 'event_update')
+
+        self.manager._event()
+
+        event_update.assert_not_called()
+
     def test_exec_event_success(self):
         event = {'id': '111-222-333',
                  'event_type': 'start_lease',
@@ -656,7 +682,17 @@ class ServiceTestCase(tests.TestCase):
             'trust_id': 'exxee111qwwwwe'}
 
         self.assertRaises(
-            exceptions.NotAuthorized, self.manager.create_lease, lease_values)
+            exceptions.InvalidInput, self.manager.create_lease, lease_values)
+
+    def test_create_lease_end_before_start(self):
+        lease_values = {
+            'name': 'lease-name',
+            'start_date': '2026-11-13 13:13',
+            'end_date': '2026-11-13 12:13',
+            'trust_id': 'exxee111qwwwwe'}
+
+        self.assertRaises(
+            exceptions.InvalidInput, self.manager.create_lease, lease_values)
 
     def test_create_lease_unsupported_resource_type(self):
         lease_values = {
@@ -894,6 +930,38 @@ class ServiceTestCase(tests.TestCase):
                 manager_ex.MissingParameter, self.manager.update_lease,
                 lease_id=self.lease_id, values=lease_values)
 
+    @ddt.data('', None, '1234', '7085381b-45e0-4e5d-b24a-f965f5e6e5d7')
+    def test_update_reservations_with_invalid_reservation_id(self,
+                                                             reservation_id):
+        lease_values = {
+            'reservations': [
+                {
+                    'disk_gb': 30,
+                    'id': reservation_id,
+                }
+            ]
+        }
+        reservation_get_all = (
+            self.patch(self.db_api, 'reservation_get_all_by_lease_id'))
+        reservation_get_all.return_value = [
+            {
+                'id': u'593e7028-c0d1-4d76-8642-2ffd890b324c',
+                'resource_type': 'virtual:instance',
+            },
+            {
+                'id': u'2eeb784a-2d84-4a89-a201-9d42d61eecb1',
+                'resource_type': 'virtual:instance',
+            }
+        ]
+        target = datetime.datetime(2013, 12, 15)
+        with mock.patch.object(datetime,
+                               'datetime',
+                               mock.Mock(wraps=datetime.datetime)) as patched:
+            patched.utcnow.return_value = target
+        self.assertRaises(
+            exceptions.InvalidInput, self.manager.update_lease,
+            lease_id=self.lease_id, values=lease_values)
+
     def test_update_lease_started_modify_end_date_without_before_end(self):
         def fake_event_get(sort_key, sort_dir, filters):
             if filters['event_type'] == 'start_lease':
@@ -1210,7 +1278,7 @@ class ServiceTestCase(tests.TestCase):
                                mock.Mock(wraps=datetime.datetime)) as patched:
             patched.utcnow.return_value = target
             self.assertRaises(
-                exceptions.NotAuthorized, self.manager.update_lease,
+                exceptions.InvalidInput, self.manager.update_lease,
                 lease_id=self.lease_id, values=lease_values)
 
     def test_update_lease_not_started_start_date_before_current_time(self):
@@ -1224,7 +1292,7 @@ class ServiceTestCase(tests.TestCase):
                                mock.Mock(wraps=datetime.datetime)) as patched:
             patched.utcnow.return_value = target
             self.assertRaises(
-                exceptions.NotAuthorized, self.manager.update_lease,
+                exceptions.InvalidInput, self.manager.update_lease,
                 lease_id=self.lease_id, values=lease_values)
 
     def test_update_lease_end_date_before_current_time(self):
@@ -1238,7 +1306,7 @@ class ServiceTestCase(tests.TestCase):
                                mock.Mock(wraps=datetime.datetime)) as patched:
             patched.utcnow.return_value = target
             self.assertRaises(
-                exceptions.NotAuthorized, self.manager.update_lease,
+                exceptions.InvalidInput, self.manager.update_lease,
                 lease_id=self.lease_id, values=lease_values)
 
     def test_update_lease_completed_modify_dates(self):
@@ -1252,7 +1320,7 @@ class ServiceTestCase(tests.TestCase):
                                mock.Mock(wraps=datetime.datetime)) as patched:
             patched.utcnow.return_value = target
             self.assertRaises(
-                exceptions.NotAuthorized, self.manager.update_lease,
+                exceptions.InvalidInput, self.manager.update_lease,
                 lease_id=self.lease_id, values=lease_values)
 
     def test_update_lease_start_date_event_not_found(self):
