@@ -16,10 +16,13 @@ import uuid as uuidgen
 
 from keystoneauth1 import session
 from keystoneauth1 import token_endpoint
+import mock
 from novaclient import client as nova_client
 from novaclient import exceptions as nova_exceptions
+from novaclient.v2 import availability_zones
 from novaclient.v2 import hypervisors
 from oslo_config import cfg
+from oslo_config import fixture
 
 from blazar import context
 from blazar.manager import exceptions as manager_exceptions
@@ -57,9 +60,10 @@ class TestCNClient(tests.TestCase):
         user_domain = 'User_Domain'
         project_name = 'admin'
         project_domain = 'Project_Domain'
-        auth_url = "%s://%s:%s/v3" % (CONF.os_auth_protocol,
+        auth_url = "%s://%s:%s/%s" % (CONF.os_auth_protocol,
                                       CONF.os_auth_host,
-                                      CONF.os_auth_port)
+                                      CONF.os_auth_port,
+                                      CONF.os_auth_prefix)
 
         kwargs = {'version': self.version,
                   'endpoint_override': endpoint,
@@ -121,6 +125,13 @@ class ReservationPoolTestCase(tests.TestCase):
         self.blazar_owner = nova_conf.blazar_owner
         self.blazar_az_prefix = physical_host_conf.blazar_az_prefix
 
+        self.cfg = self.useFixture(fixture.Config(CONF))
+        self.cfg.config(os_admin_username='fake-user')
+        self.cfg.config(os_admin_password='fake-passwd')
+        self.cfg.config(os_admin_user_domain_name='fake-user-domain')
+        self.cfg.config(os_admin_project_name='fake-pj-name')
+        self.cfg.config(os_admin_project_domain_name='fake-pj-domain')
+
         self.fake_freepool = AggregateFake(i=456,
                                            name=self.freepool_name,
                                            hosts=['host3'])
@@ -139,9 +150,16 @@ class ReservationPoolTestCase(tests.TestCase):
         self.p_name = self.patch(self.pool, '_generate_aggregate_name')
         self.p_name.return_value = self.pool_name
 
+    def test_configuration(self):
+        self.assertEqual("fake-user", self.pool.username)
+        self.assertEqual("fake-passwd", self.pool.password)
+        self.assertEqual("fake-user-domain", self.pool.user_domain_name)
+        self.assertEqual("fake-pj-name", self.pool.project_name)
+        self.assertEqual("fake-pj-domain", self.pool.project_domain_name)
+
     def _patch_get_aggregate_from_name_or_id(self):
         def get_fake_aggregate(*args):
-            if self.freepool_name in args:
+            if self.freepool_name in args or self.fake_freepool.id in args:
                 return self.fake_freepool
             else:
                 return self.fake_aggregate
@@ -269,7 +287,7 @@ class ReservationPoolTestCase(tests.TestCase):
         check0 = self.nova.aggregates.add_host
         check0.assert_any_call(self.fake_aggregate.id, 'host3')
         check1 = self.nova.aggregates.remove_host
-        check1.assert_any_call(self.fake_aggregate.id, 'host3')
+        check1.assert_any_call(self.fake_freepool.id, 'host3')
 
     def test_add_computehost_with_host_id(self):
         # NOTE(sbauza): Freepool.hosts only contains names of hosts, not UUIDs
@@ -318,6 +336,24 @@ class ReservationPoolTestCase(tests.TestCase):
         check = self.nova.aggregates.add_host
         check.assert_called_once_with(self.fake_freepool.id, 'host2')
 
+    def test_add_computehost_revert(self):
+        self._patch_get_aggregate_from_name_or_id()
+        self.fake_freepool.hosts = ['host1', 'host2']
+        self.assertRaises(manager_exceptions.HostNotInFreePool,
+                          self.pool.add_computehost,
+                          'pool', ['host1', 'host2', 'host3'])
+
+        check0 = self.nova.aggregates.add_host
+        check0.assert_has_calls([mock.call(self.fake_aggregate.id, 'host1'),
+                                 mock.call(self.fake_aggregate.id, 'host2'),
+                                 mock.call('freepool', 'host1'),
+                                 mock.call('freepool', 'host2')])
+        check1 = self.nova.aggregates.remove_host
+        check1.assert_has_calls([mock.call(self.fake_freepool.id, 'host1'),
+                                 mock.call(self.fake_freepool.id, 'host2'),
+                                 mock.call(self.fake_aggregate.id, 'host1'),
+                                 mock.call(self.fake_aggregate.id, 'host2')])
+
     def test_remove_computehost_from_freepool(self):
         self._patch_get_aggregate_from_name_or_id()
         self.pool.remove_computehost(self.freepool_name, 'host3')
@@ -378,12 +414,10 @@ class ReservationPoolTestCase(tests.TestCase):
 
     def test_remove_computehosts_with_duplicate_host(self):
         self._patch_get_aggregate_from_name_or_id()
-        self.nova.aggregates.add_host.side_effect = (
-            nova_exceptions.Conflict(409))
-        self.assertRaises(manager_exceptions.CantAddHost,
-                          self.pool.remove_computehost,
-                          'pool',
-                          'host3')
+        add_host = self.nova.aggregates.add_host
+
+        self.pool.remove_computehost('pool', 'host3')
+        add_host.assert_not_called()
 
     def test_get_computehosts_with_correct_pool(self):
         self._patch_get_aggregate_from_name_or_id()
@@ -412,6 +446,7 @@ class FakeNovaHypervisors(object):
 
     class FakeHost(object):
         id = 1
+        availability_zone = 'fake_az1'
         hypervisor_hostname = 'fake_name'
         vcpus = 1
         cpu_info = 'fake_cpu'
@@ -446,6 +481,7 @@ class FakeNovaHypervisors(object):
     @classmethod
     def expected(cls):
         return {'id': cls.FakeHost.id,
+                'availability_zone': cls.FakeHost.availability_zone,
                 'hypervisor_hostname': cls.FakeHost.hypervisor_hostname,
                 'service_name': cls.FakeHost.service['host'],
                 'vcpus': cls.FakeHost.vcpus,
@@ -456,6 +492,30 @@ class FakeNovaHypervisors(object):
                 'local_gb': cls.FakeHost.local_gb}
 
 
+class FakeAvailabilityZones(object):
+
+    class FakeAZ1(object):
+        zoneName = 'fake_az1'
+        hosts = {
+            "fake_name": {
+                "nova-compute": {}
+            },
+        }
+
+    class FakeAZ2(object):
+        zoneName = 'fake_az2'
+        hosts = {
+            "fake_name": {
+                "nova-conductor": {},
+                "nova-scheduler": {}
+            },
+        }
+
+    @classmethod
+    def list(cls, detailed=False):
+        return [cls.FakeAZ1, cls.FakeAZ2]
+
+
 class NovaInventoryTestCase(tests.TestCase):
     def setUp(self):
         super(NovaInventoryTestCase, self).setUp()
@@ -470,6 +530,9 @@ class NovaInventoryTestCase(tests.TestCase):
         self.hypervisors_search = self.patch(hypervisors.HypervisorManager,
                                              'search')
         self.hypervisors_search.side_effect = FakeNovaHypervisors.search
+        self.availability_zones = self.patch(
+            availability_zones.AvailabilityZoneManager, 'list')
+        self.availability_zones.side_effect = FakeAvailabilityZones.list
 
     def test_get_host_details_with_host_id(self):
         host = self.inventory.get_host_details('1')
@@ -494,12 +557,24 @@ class NovaInventoryTestCase(tests.TestCase):
                           self.inventory.get_host_details, 'wrong_name')
 
     def test_get_host_details_with_invalid_host(self):
-        invalid_host = FakeNovaHypervisors.FakeHost
+        # Create a new class from FakeHost called `invalid_host`,
+        # which lacks the vcpus attribute.
+        invalid_host = type('invalid_host',
+                            FakeNovaHypervisors.FakeHost.__bases__,
+                            dict(FakeNovaHypervisors.FakeHost.__dict__))
         del invalid_host.vcpus
-        self.hypervisors_get.return_value = invalid_host
+        self.hypervisors_get.side_effect = [invalid_host]
         self.assertRaises(manager_exceptions.InvalidHost,
                           self.inventory.get_host_details, '1')
 
+    def test_get_host_details_without_az(self):
+        conf = fixture.Config(CONF)
+        conf.config(group='nova', az_aware=False)
+        host = self.inventory.get_host_details('fake_name')
+        expected = FakeNovaHypervisors.expected()
+        expected['availability_zone'] = ''
+        self.assertEqual(expected, host)
+
     def test_get_servers_per_host(self):
         servers = self.inventory.get_servers_per_host('fake_name')
         self.assertEqual(FakeNovaHypervisors.FakeHost.servers, servers)
